# file MASS/negbin.q
# copyright (C) 1994-9 W. N. Venables and B. D. Ripley
#
invisible(setOldClass(c("negbin", "glm", "lm")))
invisible(setOldClass(c("summary.negbin", "summary.glm")))

"anova.negbin"<-
function(object, ..., test = "Chisq")
{
  dots <- list(...)
  if(length(dots) == 0) {
    warning("tests made without re-estimating theta")
    object$call[[1]] <- as.name("glm")
    if(is.null(object$link))
      object$link <- as.name("log")
    object$call$family <- call("negative.binomial", theta = object$
                               theta, link = object$link)
    anova.glm(object, test = test)
  } else {
    if(test != "Chisq")
      warning("only Chi-squared LR tests are implemented")
    mlist <- list(object, ...)
    nt <- length(mlist)
    dflis <- sapply(mlist, function(x) x$df.resid)
    s <- sort.list(-dflis)
    mlist <- mlist[s]
    if(any(!sapply(mlist, inherits, "negbin")))
      stop("not all objects are of class `negbin'")
    rsp <- unique(sapply(mlist, function(x) paste(formula(x)[2])))
    mds <- sapply(mlist, function(x) paste(formula(x)[3]))
    ths <- sapply(mlist, function(x) x$theta)
    dfs <- dflis[s]
    lls <- sapply(mlist, function(x) x$twologlik)
    tss <- c("", paste(1:(nt - 1), 2:nt, sep = " vs "))
    df <- c(NA,  - diff(dfs))
    x2 <- c(NA, diff(lls))
    pr <- c(NA, 1 - pchisq(x2[-1], df[-1]))
    out <- data.frame(Model = mds, theta = ths, Resid.df = dfs, 
                      "2 x log-lik." = lls, Test = tss, df = df, LRtest = x2, 
                      Prob = pr)
    names(out) <- c("Model", "theta", "Resid. df", 
                    "   2 x log-lik.", "Test", "   df", "LR stat.", "Pr(Chi)")
    oldClass(out) <- c("anova", "data.frame")
    attr(out, "heading") <-
      c("Likelihood ratio tests of Negative Binomial Models\n",
        paste("Response:", rsp))
    out
  }
}

"family.negbin"<-
function(object)
eval(substitute(negative.binomial(theta = object$theta, link = lnk), list(lnk
	 = object$call$link)))

"glm.convert"<-
function(object)
{
  object$call[[1]] <- as.name("glm")
  if(is.null(object$link))
    object$link <- as.name("log")
  object$call$family <- call("negative.binomial", theta = object$theta, 
                             link = object$link)
  object$call$init.theta <- object$call$link <- NULL
  oldClass(object) <- c("glm", "lm")
  object
}

glm.nb <- function(formula = formula(data), data = sys.parent(), weights, 
		   subset, na.action, start = eta, 
		   control = glm.control(...), method = "glm.fit", 
		   model = F, x = F, y = T, contrasts = NULL, 
		   init.theta, link = log, ...)
{
  loglik <- function(n, th, mu, y)
    {
      sum(lgamma(th + y) - lgamma(th) + th * log(th) + 
	  y * log(mu + (y == 0)) - (th + y) * log(th + mu))
    }
  link <- substitute(link)
  th <- as.name("th")
  fam <- call("negative.binomial", theta = th, link = link)
  if(missing(init.theta)) {
    fam0 <- call("poisson", link = link)
  } else {
    fam0 <- call("negative.binomial", theta = init.theta, link = link)
  }
  Call <- match.call()
  m <- match.call(expand = F)
  m$method <- m$model <- m$x <- m$y <- m$control <- m$contrasts <-
    m$init.theta <- m$link <- m$... <- NULL
  m[[1]] <- as.name("model.frame")
  m <- eval(m, sys.parent())
  Terms <- attr(m, "terms")
  if(method == "model.frame") return(m)
  a <- attributes(m)
  Y <- model.extract(m, response)
  X <- model.matrix(Terms, m, contrasts)
  w <- model.extract(m, weights)
  if(!length(w)) w <- rep(1, nrow(m))
  else if(any(w < 0)) stop("negative weights not allowed")
  start <- model.extract(m, start)
  offset <- model.extract(m, offset)
  n <- length(Y)
  if(!is.null(method)) {
    if(!existsFunction(method))
      stop(paste("unimplemented method:", method))
  }
  else method <- "glm.fit"
  glm.fitter <- get(method)
  if(control$trace > 1) cat("Initial fit:\n")
  fit <- glm.fitter(x = X, y = Y, w = w, start = start, 
		    offset = offset, family = fam0, maxit = control$
		    maxit, epsilon = control$epsilon, trace = 
		    control$trace > 1, null.dev = T, ...)
  oldClass(fit) <- c("glm", "lm")
  mu <- fitted(fit)
  th <- as.vector(theta.ml(Y, mu, n, limit=control$maxit, trace = 
                           control$trace> 2))
  if(control$trace > 1)
    cat("Initial value for theta:", signif(th), "\n")
  iter <- 0
  d1 <- sqrt(2 * max(1, fit$df.residual))
  d2 <- del <- 1
  g <- eval(fam)$link
  Lm <- loglik(n, th, mu, Y)
  Lm0 <- Lm + 2 * d1
  while((iter <- iter + 1) <= control$maxit && 
	(abs(Lm0 - Lm)/d1 + abs(del)/d2) > control$epsilon) {
    eta <- g(mu)
    fit <- glm.fitter(x = X, y = Y, w = w, start = 
		      eta, offset = offset, family = eval(fam),
		      maxit = control$maxit, epsilon = control$epsilon, 
		      trace = control$trace > 1, null.dev = T, ...)
    t0 <- th
    th <- theta.ml(Y, mu, n, limit=control$maxit, trace = control$trace > 2)
    oldClass(fit) <- c("negbin", "glm", "lm")
    mu <- fitted(fit)
    del <- t0 - th
    Lm0 <- Lm
    Lm <- loglik(n, th, mu, Y)
    if(control$trace) {
      Ls <- loglik(n, th, Y, Y)
      Dev <- 2 * (Ls - Lm)
      cat("Theta(", iter, ") =", signif(th), 
	  ", 2(Ls - Lm) =", signif(Dev), "\n")
    }
  }
  if(!is.null(attr(th, "warn"))) fit$th.warn <- attr(th, "warn")
  if(iter > control$maxit) {
    warning("alternation limit reached")
    fit$th.warn <- "alternation limit reached"
  }
  
  # If an offset and intercept are present, iterations are needed to
  # compute the Null deviance; these are done here, unless the model
  # is NULL, in which case the computations have been done already
  #
  if(any(offset) && attr(Terms, "intercept")) {
    null.deviance <-
      if(length(Terms))
        glm.fitter(X[, "(Intercept)", drop = F], Y, w,
                   offset = offset, family = eval(fam),
                   maxit = control$maxit,
                   epsilon = control$epsilon, null.dev = NULL)$deviance
      else fit$deviance
    fit$null.deviance <- null.deviance
  }
  oldClass(fit) <- c("negbin", "glm", "lm")
  fit$terms <- Terms
  fit$formula <- as.vector(attr(Terms, "formula"))
  Call$init.theta <- as.vector(th)
  Call$link <- link
  fit$call <- Call
  if(model) fit$model <- m
  if(x) fit$x <- X
  if(!y) fit$y <- NULL
  fit$theta <- as.vector(th)
  fit$SE.theta <- attr(th, "SE")
  fit$twologlik <- as.vector(2 * Lm)
  fit
}

"negative.binomial" <-
function(theta = stop("theta must be specified"), link = "log")
{
  link <- as.character(substitute(link))
  misnames <- c("log", "identity", "sqrt", "Log", "Identity", "Sqrt")
  corresp <- c(1, 2, 3, 1, 2, 3)
  lmatch <- pmatch(link, misnames, F)
  if(!lmatch)
    stop(paste("Negative binomial links are currently only",
	       "\"log\", \"identity\", or\n\t  \"sqrt\"."))
  link <- misnames[corresp[lmatch]]
  var <- list(name = paste("mu + mu^2", format(round(theta, 4)), sep = "/"), 
	      variance = substitute(function(mu) mu * (1 + mu/.theta), 
                list(.theta = theta)), 
	      deviance = substitute(function(mu, y, w, residuals = F) {
		devi <- 2 * w * (y * log(pmax(1, y)/mu) - (y + .theta) *
                                 log((y + .theta)/(mu + .theta)))
		if(residuals) sign(y - mu) * sqrt(abs(devi))
		else sum(devi)
	      }
                , list(.theta = theta)))
  make.family("Negative Binomial", glm.links[, link], var)
}

rnegbin <-
function(n, mu = n, theta = stop("theta must be given"))
{
  k <- if(length(n) > 1) length(n) else n
  rpois(k, (mu * rgamma(k, theta))/theta)
}

"summary.negbin" <-
function(object, dispersion = 1, correlation = T, ...)
{
  if(is.null(dispersion)) dispersion <- 1
  summ <- c(summary.glm(object, dispersion = dispersion, 
                        correlation = correlation), 
            object[c("theta", "SE", "twologlik", "th.warn")])
  oldClass(summ) <- c("summary.negbin", "summary.glm")
  summ
}

"print.summary.negbin" <- function(x, ...)
{
  NextMethod()
  dp <- 2 - floor(log10(x$SE))
  cat("\n              Theta: ", format(round(x$theta, dp), sci=F, nsmall=dp),
      "\n          Std. Err.: ", format(round(x$SE, dp), sci=F, nsmall=dp),
      "\n") 
  if(!is.null(x$th.warn))
    cat("Warning while fitting theta:", x$th.warn,"\n")
  cat("\n 2 x log-likelihood: ", format(round(x$twologlik, 3), sci=F, nsmall=3), "\n")
  invisible(x)
}

"theta.md" <-
function(y, u, dfr, limit = 20, eps = sqrt(.Machine$single.eps))
{
  if(inherits(y, "lm")) {
    u <- fitted(y)
    dfr <- y$df.residual
    y <- if(is.null(y$y)) u + residuals(y) else y$y
  }
  n <- length(y)
  t0 <- n/sum((y/u - 1)^2)
  a <- 2 * sum(y * log(pmax(1, y)/u)) - dfr
  it <- 0
  del <- 1
  while((it <- it + 1) < limit && abs(del) > eps) {
    t0 <- abs(t0)
    top <- a - 2 * sum((y + t0) * log((y + t0)/(u + t0)))
    bot <- 2 * sum((y - u)/(u + t0) - log((y + t0)/(u + t0)))
    del <- top/bot
    t0 <- t0 - del
  }
  if(t0 < 0) {
    t0 <- 0
    warning("estimator truncated at zero")
    attr(t0, "warn") <- "estimate truncated at zero"
  }
  t0
}

"theta.ml" <-
function(y, mu, n = length(y), limit = 10, eps = sqrt(.Machine$single.eps),
         trace=F)
{
  score <- function(n, th, mu, y)
    sum(digamma(th + y) - digamma(th) + log(th) + 
	1 - log(th + mu) - (y + th)/(mu + th))
  info <- function(n, th, mu, y)
    sum( - trigamma(th + y) + trigamma(th) - 1/th + 
	2/(mu + th) - (y + th)/(mu + th)^2)
  if(inherits(y, "lm")) {
    mu <- fitted(y)
    y <- if(is.null(y$y)) mu + residuals(y) else y$y
  }
  t0 <- n/sum((y/mu - 1)^2)
  it <- 0
  del <- 1
  if(trace) cat("theta.ml: initial theta =", signif(t0), "\n")
  while((it <- it + 1) < limit && abs(del) > eps) {
    t0 <- abs(t0)
    del <- score(n, t0, mu, y)/(i <- info(n, t0, mu, y))
    t0 <- t0 + del
  if(trace) cat("theta.ml: iter", it," theta =", signif(t0), "\n")
  }
  if(t0 < 0) {
    t0 <- 0
    warning("estimator truncated at zero")
    attr(t0, "warn") <- "estimate truncated at zero"
  }
  if(it == limit) {
    warning("iteration limit reached")
    attr(t0, "warn") <- "iteration limit reached"
  }
  attr(t0, "SE") <- sqrt(1/i)
  t0
}

"theta.mm" <-
function(y, u, dfr, limit = 10, eps = sqrt(.Machine$single.eps))
{
  if(inherits(y, "lm")) {
    u <- fitted(y)
    dfr <- y$df.residual
    y <- if(is.null(y$y)) u + residuals(y) else y$y
  }
  n <- length(y)
  t0 <- n/sum((y/u - 1)^2)
  it <- 0
  del <- 1
  while((it <- it + 1) < limit && abs(del) > eps) {
    t0 <- abs(t0)
    del <- (sum((y - u)^2/(u + u^2/t0)) - dfr)/sum((y - u)^2/(u + t0)^2)
    t0 <- t0 - del
  }
  if(t0 < 0) {
    t0 <- 0
    warning("estimator truncated at zero")
    attr(t0, "warn") <- "estimate truncated at zero"
  }
  t0
}
