.\" file MASS/qda.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN qda
.TL
Quadratic Discriminant Analysis
.DN
Quadratic discriminant analysis.
.CS
qda(formula, data, prior = proportions, subset, 
                   na.action = na.fail, method, CV = F, nu)
qda(x,   grouping, prior = proportions, subset,
                   na.action = na.fail, method, CV = F, nu)
.RA
.AG formula
A formula of the form `groups ~ x1 + x2 + ...'  That is, the
response is the grouping factor and the right hand side specifies
the (non-factor) discriminators.
.OA
.AG data
Data frame from which variables specified in `formula' are
preferentially to be taken.
.AG x
(required if no formula is given as the principal argument.)
a matrix or data frame or Matrix containing the explanatory variables.
.AG grouping
(required if no formula principal argument is given.)
a factor specifying the class for each observation.
.AG prior
the prior probabilities of class membership.  If unspecified, the class
proportions for the training set are used.  If specified, the
probabilities should be specified in the order of the factor levels.
.AG subset
An index vector specifying the cases to be used in the training
sample.  (NOTE: If given, this argument must be named.)
.AG na.action
A function to specify the action to be taken if `NA's are found.
The default action is for the procedure to fail.  An alternative is
na.omit, which leads to rejection of cases with missing values on
any required variable.  (NOTE: If given, this argument must be named.)
.AG method
`"moment"' for standard estimators of the mean and variance,
`"mle"' for MLEs, `"mve"' to use `cov.mve', or `"t"' for robust 
estimates based on a t distribution.
.AG CV
If true, returns results (classes and posterior probabilities) for
leave-out-out cross-validation. Note that if the prior is estimated,
the proportions in the whole dataset are used.
.AG nu
degrees of freedom for `method = "t"'.
.RT
an object of class qda containing the following components:
.RC prior
the prior probabilities used.
.RC means
the group means.
.RC scaling
for each group `i', `scaling[,,i]' is an array which transforms observations
so that within-groups covariance matrix is spherical.
.RC ldet
a vector of half log determinants of the dispersion matrix.
.RC lev
the levels of the grouping factor.
.RC terms
(if formula is a formula) 
an object of mode expression and class term summarizing
the  formula.
.RC call
the (matched) function call.
.PP
unless `CV=T', when the return value is a list with components:
.RC class
The MAP classification (a factor)
.RC posterior
posterior probabilities for the classes
.DT
Uses a QR decomposition which will give an error message if the
within-group variance is singular for any group.
.SA
`predict.qda', `lda'
.EX
tr <- sample(1:50,25)
train <- rbind(iris[tr,,1],iris[tr,,2],iris[tr,,3])
test <- rbind(iris[-tr,,1],iris[-tr,,2],iris[-tr,,3])
cl <- factor(c(rep("s",25),rep("c",25), rep("v",25)))
z <- qda(train, cl)
predict(z,test)$class
.KW multivariate
.WR
