.\" file class/lvqinit.d copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN lvqinit
.TL
Initialize a LVQ Codebook
.DN
Construct an initial codebook for LVQ methods.
.CS
lvqinit(x, cl, size, prior, k)
.RA
.AG x
a matrix or data frame of training examples, `n' by `p'.
.AG cl
the classifications for the training examples. A vector or factor of
length `n'.
.OA
.AG size
the size of the codebook. Defaults to `min(round(0.4*ng*(ng-1 + p/2),0), n)'
where `ng' is the number of classes.
.AG prior
Probabilities to represent classes in the codebook. Default proportions in the
training set.
.AG k
k used for k-NN test of correct classification. Default is 5.
.RT
A codebook, represented as a list with components `x' and `cl' giving
the examples and classes.
.DT
Selects `size' examples from the training set without replacement with
proportions proportional to the prior or the original proportions.
.SH REFERENCES
Kohonen, T. (1990) The self-organizing map.  
.ul
Proc. IEEE 
\fB78\fR, 1464-1480.
.sp
Kohonen, T. (1995)
.ul
Self-Organizing Maps.
Springer, Berlin.
.SA
`lvq1', `lvq2', `lvq3', `olvq1', `lvqtest'
.EX
train <- rbind(iris[1:25,,1],iris[1:25,,2],iris[1:25,,3])
test <- rbind(iris[26:50,,1],iris[26:50,,2],iris[26:50,,3])
cl <- factor(c(rep("s",25),rep("c",25), rep("v",25)))
cd <- lvqinit(train, cl, 10)
lvqtest(cd, train)
cd1 <- olvq1(train, cl, cd)
lvqtest(cd1, train)
.KW classif
.WR
