.\" file class/multiedit.d copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN multiedit
.TL
Multiedit for k-NN Classifier
.DN
Multiedit for k-NN classifier
.CS
multiedit(x, class, k=1, V=3, I=5, trace=T)
.RA
.AG x
matrix of training set.
.AG class
vector of classification of training set.
.AG k
number of neighbours used in k-NN.
.OA
.AG V
divide training set into V parts.
.AG I
number of null passes before quitting.
.AG trace
logical for statistics at each pass.
.RT
index vector of cases to be retained.
.SH REFERENCE
P. A. Devijver and J. Kittler (1982)
.ul
Pattern Recognition. A Statistical Approach.
Prentice-Hall, p. 115.
.SA
`condense', `reduce.nn'
.EX
set.seed(99)
tr <- sample(1:50,25)
train <- rbind(iris[tr,,1],iris[tr,,2],iris[tr,,3])
test <- rbind(iris[-tr,,1],iris[-tr,,2],iris[-tr,,3])
cl <- factor(c(rep(1,25),rep(2,25), rep(3,25)), labels=c("s", "c", "v"))
table(cl, knn(train, test, cl, 3))
ind1 <- multiedit(train, cl, 3)
length(ind1)
table(cl, knn(train[ind1,], test, cl[ind1], 1))
ntrain <- train[ind1,]; ncl <- cl[ind1]
ind2 <- condense(ntrain, ncl)
length(ind2)
table(cl, knn(ntrain[ind2,], test, ncl[ind2], 1))
.KW classif
.WR
