.\" file nnet/nnet.d copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN nnet
.FN nnet.default
.FN nnet.formula
.TL
Fit Neural Networks
.DN
Fit single-hidden-layer neural network, possibly with skip-layer connections.
.CS
nnet.formula(formula, data=NULL, ...,
             subset, na.action=na.fail, contrasts=NULL)
nnet.default(x, y, weights, size, Wts, mask,
             linout=F, entropy=F, softmax=F, censored=F,
             skip=F, rang=0.7, decay=0, maxit=100,
             Hess=F, trace=T, MaxNWts=1000, abstol=1.0e-4, reltol=1.0e-4)
.RA
.AG formula
A formula of the form `class ~ x1 + x2 + ...' 
.AG x
matrix or data frame of `x' values for examples.
.AG y
matrix or data frame of target values for examples.
.AG weights
(case) weights for each example \(em if missing defaults to 1.
.AG size
number of units in the hidden layer. Can be zero if there are skip-layer units.
.OA
.AG data
Data frame from which variables specified in  `formula' are
preferentially to be taken.
.AG subset
An index vector specifying the cases to be used in the training
sample.  (NOTE: If given, this argument must be named.)
.AG na.action
A function to specify the action to be taken if `NA's are found.
The default action is for the procedure to fail.  An alternative is
na.omit, which leads to rejection of cases with missing values on
any required variable.  (NOTE: If given, this argument must be named.)
.AG contrasts
a list of contrasts to be used for some or all  of
the  factors  appearing as variables in the model formula.
.AG Wts
initial parameter vector. If missing chosen at random.
.AG mask
logical vector indicating which parameters should be optimized (default all).
.AG linout
switch for linear output units. Default logistic output units.
.AG entropy
switch for entropy (= maximum conditional likelihood) fitting. 
Default by least-squares. 
.AG softmax
switch for softmax (log-linear model) and maximum conditional
likelihood fitting. `linout', `entropy', `softmax' and `censored' are mutually
exclusive.
.AG censored
A variant on `softmax', in which non-zero targets mean possible
classes. Thus for `softmax' a row of `(0, 1, 1)' means one example
each of classes 2 and 3, but for `censored' it means one example whose
class is only known to be 2 or 3.
.AG skip
switch to add skip-layer connections from input to output.
.AG rang
Initial random weights on [-`rang', `rang'].  Value about 0.5 unless the 
inputs are large, in which case it should be chosen so that 
`rang' * max(`|x|') is about 1.
.AG decay
parameter for weight decay.  Default 0.
.AG maxit
maximum number of iterations. Default 100.
.AG Hess
If true, the Hessian of the measure of fit at the best set of weights
found is returned as component `Hessian'.
.AG trace
switch for tracing optimization. Default `T'.
.AG MaxNWts
The maximum allowable number of weights.  There is no intrinsic limit
in the code, but increasing `MaxNWts' will probably allow fits that
are very slow and time-consuming (and perhaps uninterruptable under
Windows).
.AG abstol
Stop if the fit criterion falls below `abstol', indicating an
essentially perfect fit.
.AG reltol
Stop if the optimizer is unable to reduce the fit criterion by a
factor of at least `1 - reltol'.
.RT
object of class `nnet' or `nnet.formula'.
Mostly internal structure, but has components
.RC wts
the best set of weights found
.RC value
value of fitting criterion plus weight decay term.
.RC fitted.values
the fitted values for the training data.
.RC residuals
the residuals for the training data.
.DT
If the response in `formula' is a factor, an appropriate classification
network is constructed; this has one output and entropy fit if the
number of levels is two, and a number of outputs equal to the number
of classes and a softmax output stage for more levels.  If the
response is not a factor, it is passed on unchanged to `nnet.default'.
.PP
A quasi-Newton optimizer is used, written in `C'.
.SA
`predict.nnet', `nnet.Hess'
.EX
# use half the iris data
ir <- rbind(iris[,,1],iris[,,2],iris[,,3])
targets <- class.ind( c(rep("s", 50), rep("c", 50), rep("v", 50)) )
samp <- c(sample(1:50,25), sample(51:100,25), sample(101:150,25))
ir1 <- nnet(ir[samp,], targets[samp,], size=2, rang=0.1, 
            decay=5e-4, maxit=200)
test.cl <- function(true, pred){
        true <- max.col(true)
        cres <- max.col(pred)
        table(true, cres)
}
test.cl(targets[-samp,], predict(ir1, ir[-samp,]))

# or
ird <- data.frame(rbind(iris[,,1], iris[,,2], iris[,,3]),
	species=c(rep("s",50), rep("c", 50), rep("v", 50)))
ir.nn2 <- nnet(species ~ ., data=ird, subset=samp, size=2, rang=0.1, 
            decay=5e-4, maxit=200)
table(ird$species[-samp], predict(ir.nn2, ird[-samp,], type="class"))
.KW neural
.WR
