# file treefix/treefix.q copyright (C) 1994-9 B. D. Ripley
#
# Based originally on code from S-PLUS 3.3, copyright (C) 1995 StatSci Inc.
# The code in S-PLUS 3.4 and later is based on versions of this code.
#
# May be used only with a licensed copy of S-PLUS 3.2 or later.
#
prune.tree <- function(tree, k=NULL, best=NULL, newdata, nwts,
		       method=c("deviance", "misclass"),
		       loss=1-diag(nc), eps=1e-3)
{
  if(inherits(tree, "singlenode")) stop("Can't prune singlenode tree")
  if(!inherits(tree, "tree")) stop("Not legitimate tree")
  method <- match.arg(method)
  nc <- length(attr(tree, "ylevels"))
  if(method == "misclass" & !nc)
    stop("misclass only for classification trees")
  frame <- tree$frame
  node <- row.names(frame)
  nodes <- as.numeric(node)
  nnode <- length(node)
  ndim <- ceiling(nnode/2)

  if(is.null(y <- tree$y))
    y <- model.extract(model.frame.tree(tree), "response")
  if(is.null(w <- tree$call$weights)) w <- rep(1, length(y))
  else w <- model.extract(model.frame.tree(tree), "weights")
  if(method == "misclass") {
    Z <- .C("VR_dev1",
	    nnode, nodes, integer(nnode),
	    dev = double(nnode), sdev = double(nnode),
	    y, length(y), frame$yval, tree$where, w, nc, loss,
            CLASSES = c(rep("integer",3), rep("double", 2),
              rep("integer", 4), "double", "integer", "double"),
            COPY = rep(F, 12)
	    )
    dev <- Z$dev; sdev <- Z$sdev
  } else {
    dev <- tree$frame$dev
    if(!nc) {
      sdev <- .C("VR_dev3",
		 nnode, nodes, integer(nnode),
		 dev = double(nnode), sdev = double(nnode),
		 y, length(y), frame$yval, tree$where, w,
                 CLASSES = c(rep("integer",3), rep("double", 3),
                   "integer", "double", "integer", "double"),
                 COPY = rep(F, 10)
		 )$sdev
    } else  {
      sdev <- -2 * .C("VR_dev2",
		      nnode, nodes, integer(nnode),
		      dev = double(nnode), sdev = double(nnode),
		      y, length(y), frame$yprob, tree$where, w,
                      CLASSES = c(rep("integer",3), rep("double", 2),
                        rep("integer", 2), "double", "integer", "double"),
                      COPY = rep(F, 10)
		      )$sdev
    }
  }
  if(missing(newdata) || is.null(newdata)) {
    ndev <- dev
    nsdev <- sdev
  } else {
    if(is.null(attr(newdata, "terms")))
      nd <- model.frame(tree$terms, newdata, na.action=na.pass)
    else nd <- newdata
    y <- model.extract(nd, "response")
    if(missing(nwts)) nwts <- rep(1, length(y))
    where <- pred1.tree(tree, tree.matrix(nd))
    if(method == "misclass") {
      Z <- .C("VR_dev1",
              nnode, nodes, integer(nnode),
              dev = double(nnode), sdev = double(nnode),
              y, length(y), frame$yval, where, w, nc, loss,
              CLASSES = c(rep("integer",3), rep("double", 2),
                rep("integer", 4), "double", "integer", "double"),
              COPY = rep(F, 12)
	      )
      ndev <- Z$dev; nsdev <- Z$sdev
    } else {
      if(!nc) {
	Z <- .C("VR_dev3",
                nnode, nodes, integer(nnode),
                dev = double(nnode), sdev = double(nnode),
                y, length(y), frame$yval, where, nwts,
                CLASSES = c(rep("integer",3), rep("double", 3),
                  "integer", "double", "integer", "double"),
                COPY = rep(F, 10)
		)
	ndev <- Z$dev; nsdev <- Z$sdev
      } else {
	yp <- frame$yprob
	yp[yp==0] <- max(0,eps)
	Z <- .C("VR_dev2",
                nnode, nodes, integer(nnode),
                dev = double(nnode), sdev = double(nnode),
                y, length(y), yp, where, nwts,
                CLASSES = c(rep("integer",3), rep("double", 2),
                  rep("integer", 2), "double", "integer", "double"),
                COPY = rep(F, 10)
		)
	ndev <- -2 * Z$dev; nsdev <- -2 *Z$sdev
      }
    }
  }
  zp <- .C("VR_prune2",
	   n=nnode, nodes, frame$var == "<leaf>",
	   dev, sdev, ndev, nsdev, keep=integer(nnode), sort.list(nodes),
	   double(nnode), integer(nnode), double(nnode),
           alpha=double(ndim), inode=integer(ndim),
           size=integer(ndim), deviance=double(ndim), newdev=double(ndim),
           CLASSES = c(rep("integer", 3), rep("double", 4),
             rep("integer", 2), "double", "integer", rep("double", 2),
             rep("integer", 2), rep("double", 2)),
           COPY = c(T, rep(F, 16))
	   )
  n <- zp$n
  alpha <- zp$alpha[1:n]
  #  alpha[1] <- -Inf
  size <- zp$size[1:n]
  index <- 0
  if(missing(k) || is.null(k)) {
    ind <- drop(outer(unique(alpha), alpha, ">=") %*% rep(1, length(alpha)))
    k <- alpha[ind]
    k[1] <- -Inf
    deviance <- zp$newdev[ind]
    size <- size[ind]
    if(!missing(best) && !is.null(best)) {
      index <- ind[sum(best <= size)]
      if(length(index) == 0) {
        warning("best is bigger than tree size")
        index <- 1
      }
    }
  } else {
    if(length(k) == 1) index <- sum(k >= alpha)
    else {
      k <- pmax(k, -1e+100)
      ind <- drop(outer(k, alpha, ">=") %*% rep(1, length(alpha)))
      deviance <- zp$newdev[ind]
      size <- size[ind]
    }
  }
  if(index == 1) return(tree)
  if(index > 1) {
    pnodes <- zp$inode[-1]
    tree <- snip.tree(tree, pnodes[seq(index-1)])
    tree$call$tree <- match.call()$tree
    return(tree)
  }
  obj <- list(size = size, dev = deviance, k = k, method = method)
  oldClass(obj) <- c("prune", "tree.sequence")
  obj
}
invisible(setOldClass(c("prune", "tree.sequence")))

predict.tree <-
  function(object, newdata = list(), 
           type = c("vector", "tree", "class", "where"),
           split=F, nwts, eps=1e-3)
{
  which.is.max <- function(x)
  {
    y <- seq(length(x))[x == max(x)]
    if(length(y) > 1) sample(y,1)
    else y
  }


  pred2.tree  <- function(tree, x)
  {
    frame <- tree$frame
    if(!length(frame$yprob)) stop("only for classification trees")
    dimx <- dim(x)
    ypred <- .C("VR_pred2",
                x, oldUnclass(frame$var) - 1, #0 denotes leaf node
                frame$splits[, "cutleft"], frame$splits[, "cutright"],
                sapply(attr(tree, "xlevels"), length),
                row.names(frame), frame$n, nf <- dim(frame)[1],
                dimx[1], where = single(dimx[1]),
                NAOK = T,
                CLASSES = c("double", "integer", rep("character",2),
                  rep("integer", 5), "single"),
                COPY = rep(F, 11)
                )
    ypred <- matrix(ypred$where, nf)
    dimnames(ypred) <- list(row.names(frame),dimnames(x)[[1]])
    ypred
  }

  if(!inherits(object, "tree") && !inherits(object, "singlenode")) 
    stop("Not legitimate tree")
  type <- match.arg(type)
  if(type == "class" && is.null(attr(object, "ylevels")))
    stop("type class only for classification trees")
  if(missing(newdata) || is.null(newdata) & type == "tree")
    return(object)                      #idiot proofing
  if(missing(newdata) || is.null(newdata)) {
    where <- object$where
    newdata <- model.frame.tree(object)
    if(!is.null(w <- object$call$weights))
      nwts <- model.extract(model.frame.tree(object), "weights")
  } else {
    if(is.null(attr(newdata, "terms"))) {
      # newdata is not a model frame.
      Terms <- object$terms
      y <- match(as.character(formula(Terms)[[2]]), names(newdata))
      if(length(y) > 1) y <- y[-1]  # was something like log(y)
      if(type != "class" || any(is.na(y))) Terms <- delete.response(Terms)
      newdata <- model.frame(Terms, newdata, na.action = na.pass)
    }
    where <- pred1.tree(object, tree.matrix(newdata))
  }
  if(type == "where") return(where)
  frame <- object$frame
  node <- row.names(frame)
  nodes <- as.numeric(node)
  nnode <- length(node)
  if(type != "tree")
    if(is.null(lev <- attr(object, "ylevels"))) {
      if(!split) {
	frame <- frame$yval[where]
	names(frame) <- names(where)
	return(frame)
      } else {
	where <- pred2.tree(object, tree.matrix(newdata))
	leaf <- frame$var=="<leaf>"
	frame <- t(where[leaf, , drop=F]) %*% frame$y[leaf]
	names(frame) <- names(where)
	return(frame)
      }
    } else {
      if(!split) {
	pr <- frame$yprob[where,  , drop = F]
	dimnames(pr)[[1]] <- names(where)
      } else {
	where <- pred2.tree(object, tree.matrix(newdata))
	leaf <- frame$var=="<leaf>"
	pr <- t(where[leaf,,drop=F]) %*% frame$yprob[leaf,,drop=F]
	dimnames(pr) <- list(names(where), lev)
      }
      if(type=="class") {
	cl <- apply(pr, 1, which.is.max)
	return(factor(lev[cl], levels=lev))
      } else return(pr)
    }
  # now must be type = "tree"
  which <- descendants(as.numeric(row.names(frame)))[, where, drop = F]
  if(any(is.na(y))) dev <- rep(NA, nrow(frame))
  else {
    y <- model.extract(newdata, "response")
    if(missing(nwts)) nwts <- rep(1, length(y))
    if(!length(attr(object, "ylevels"))) {
#      dev <- outer(frame$yval, y, "-") * which
#      dev <- (dev * dev) %*% rep(1, length(y))
#
#  handle NAs in y separately.
#
      drp <- is.na(y); nwts[drp] <- 0; y[drp] <- 0
      dev <- .C("VR_dev3",
                nnode, nodes, integer(nnode),
                dev = double(nnode), sdev = double(nnode),
                y, length(y), frame$yval, where, nwts,
                CLASSES = c(rep("integer",3), rep("double", 3),
                  "integer", "double", "integer", "double"),
                COPY = rep(F, 10)
		)$dev
      dev[which %*% drp > 0] <- NA
    } else {
#      dev <- -2 * log(1 + which * (frame$yprob[, y, drop = F] - 1)) %*% rep(1, length(y))
      yp <- frame$yprob
      yp[yp==0] <- max(0,eps)
      drp <- is.na(y); nwts[drp] <- 0; y[drp] <- levels(y)[1]
      dev <- -2 * .C("VR_dev2",
                     nnode, nodes, integer(nnode),
                     dev = double(nnode), sdev = double(nnode),
                     y, length(y), yp, where, nwts,
                     CLASSES = c(rep("integer",3), rep("double", 2),
                       rep("integer", 2), "double", "integer", "double"),
                     COPY = rep(F, 10)
                     )$dev
      dev[which %*% drp > 0] <- NA
    }
  }
  object$frame$dev <- as.vector(dev)
  object$frame$n <- as.vector(which %*% rep(1, length(where)))
  object$where <- where
  object$call <- match.call()
  object$y <- y
  object
}

pred1.tree <- function(tree, x)
{
  frame <- tree$frame
  dimx <- dim(x)
  ypred <- .C("VR_pred1",
              x, unclass(frame$var) - 1, #0 denotes leaf node
              frame$splits[, "cutleft"], frame$splits[, "cutright"],
              sapply(attr(tree, "xlevels"), length),
              row.names(frame), frame$n, dim(frame)[1],
              dimx[1], dimx[2], where = integer(dimx[1]),
              NAOK = T,
              CLASSES = c("double", "integer", rep("character",2),
                rep("integer", 7)),
              COPY = rep(F, 11)
              )
  ypred <- ypred$where
  names(ypred) <- dimnames(x)[[1]]
  ypred
}
